@@ -360,7 +360,7 @@ add_index_dbi(caddr_t attr, caddr_t otx)
     if (ai->ai_indexmask & INDEX_ANY) {
         octx->rc = add_dbi(octx, octx->be, ai->ai_type, flags);
         octx->ai = NULL;
-        return octx->rc ? STOP_AVL_APPLY : 0;
+        return octx->rc && octx->rc != MDB_NOTFOUND ? STOP_AVL_APPLY : 0;
     } else {
         octx->ai = NULL;
         return 0;
@@ -691,6 +691,7 @@ int dbmdb_make_env(dbmdb_ctx_t *ctx, int readOnly, mdb_mode_t mode)
     dbmdb_info_t infofileinfo = {0};
     dbmdb_info_t curinfo = ctx->info;
     MDB_envinfo envinfo = {0};
+    char size_buffer[10] = {0};
     int rc = 0;
 
     init_mdbtxn(ctx);
@@ -744,9 +745,12 @@ int dbmdb_make_env(dbmdb_ctx_t *ctx, int readOnly, mdb_mode_t mode)
         }
     }
 
-    slapi_log_err(SLAPI_LOG_INFO, "dbmdb_make_env", "MDB environment created with maxsize=%lu.\n", ctx->startcfg.max_size);
-    slapi_log_err(SLAPI_LOG_INFO, "dbmdb_make_env", "MDB environment created with max readers=%d.\n", ctx->startcfg.max_readers);
-    slapi_log_err(SLAPI_LOG_INFO, "dbmdb_make_env", "MDB environment created with max database instances=%d.\n", ctx->startcfg.max_dbs);
+    slapi_log_err(SLAPI_LOG_INFO, "dbmdb_make_env",
+                  "MDB environment created with maxsize=%lu (%s)\n",
+                  ctx->startcfg.max_size,
+                  convert_bytes_to_str((double)(ctx->startcfg.max_size), size_buffer, 0));
+    slapi_log_err(SLAPI_LOG_INFO, "dbmdb_make_env", "MDB environment created with max readers=%d\n", ctx->startcfg.max_readers);
+    slapi_log_err(SLAPI_LOG_INFO, "dbmdb_make_env", "MDB environment created with max database instances=%d\n", ctx->startcfg.max_dbs);
 
     /* If some upgrade is needed based on libmdb version, then another test must be done here.
      *  and the new test should be based on infofileinfo.libversion
@@ -1357,7 +1361,7 @@ int dbmdb_open_cursor(dbmdb_cursor_t *dbicur, dbmdb_ctx_t *ctx, dbmdb_dbi_t *dbi
     dbicur->dbi = dbi;
     if (ctx->readonly)
         flags |= MDB_RDONLY;
-    rc = START_TXN(&dbicur->txn, NULL, 0);
+    rc = START_TXN(&dbicur->txn, NULL, ((flags&MDB_RDONLY) ? TXNFL_RDONLY : 0));
     if (rc) {
         return rc;
     }
