@@ -37,14 +37,14 @@ void attr_create_empty(backend *be, char *type, struct attrinfo **ai);
  * cache.c
  */
 void cache_disable(void);
-int cache_init(struct cache *cache, uint64_t maxsize, int64_t maxentries, int type);
+int cache_init(struct cache *cache, struct ldbm_instance *inst, uint64_t maxsize, int64_t maxentries, int type);
 void cache_clear(struct cache *cache, int type);
 void cache_destroy_please(struct cache *cache, int type);
-void cache_set_max_size(struct cache *cache, uint64_t bytes, int type);
-void cache_set_max_entries(struct cache *cache, int64_t entries);
+void cache_set_max_size(struct cache *cache, uint64_t bytes, int type, bool autotuned);
+void cache_set_max_entries(struct cache *cache, int64_t entries, bool autotuned);
 uint64_t cache_get_max_size(struct cache *cache);
 int64_t cache_get_max_entries(struct cache *cache);
-void cache_get_stats(struct cache *cache, uint64_t *hits, uint64_t *tries, uint64_t *entries, int64_t *maxentries, uint64_t *size, uint64_t *maxsize);
+void cache_get_stats(struct cache *cache, struct cache_stats *stats);
 void cache_debug_hash(struct cache *cache, char **out);
 int cache_remove(struct cache *cache, void *e);
 void cache_return(struct cache *cache, void **bep);
@@ -326,7 +326,6 @@ void ldbm_nasty(const char *func, const char *str, int c, int err);
 void ldbm_log_access_message(Slapi_PBlock *pblock, char *string);
 int return_on_disk_full(struct ldbminfo *li);
 int ldbm_attribute_always_indexed(const char *attrtype);
-void ldbm_destroy_instance_name(struct ldbminfo *li);
 char *compute_entry_tombstone_dn(const char *entrydn, const char *uniqueid);
 char *compute_entry_tombstone_rdn(const char *entryrdn, const char *uniqueid);
 int instance_set_busy(ldbm_instance *inst);
@@ -345,6 +344,7 @@ int get_value_from_string(const char *string, char *type, char **value);
 int get_values_from_string(const char *string, char *type, char ***valuearray);
 void normalize_dir(char *dir);
 void ldbm_set_error(Slapi_PBlock *pb, int retval, int *ldap_result_code, char **ldap_result_message);
+char *convert_bytes_to_str(double bytes, char *buffer, int level);
 
 /*
  * nextid.c
@@ -372,6 +372,8 @@ const Slapi_DN *backentry_get_sdn(const struct backentry *e);
 
 struct backdn *backdn_init(Slapi_DN *sdn, ID id, int to_remove_from_hash);
 void backdn_free(struct backdn **bdn);
+void backentry_init_weight(BackEntryWeightData *starttime);
+void backentry_compute_weight(struct backentry *e, const BackEntryWeightData *starttime);
 
 /*
  * parents.c
